#version 330

layout (location = 0) out vec4 outColor;
layout (location = 1) out vec4 outDepth;

in VS_OUT {
	vec3 vsPosition;
	vec2 uv;
	vec3 tsToLight;
	vec3 tsToCamera;
} fs_in;

// material
struct Material {
	vec3 ambient;

	sampler2D diffuseMap;
	int useDiffuseMap;
	vec3 diffuse;

	sampler2D specularMap;
	int useSpecularMap;
	vec3 specular;
	float shininess;

	sampler2D normalMap;
	int useNormalMap;
};
uniform Material uMaterial;

// fog
struct Fog {
	float density;
	float gradient;
	vec3 color;
};
uniform Fog uFog;

// light
struct DirectionalLight {
	vec3 direction;
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};
uniform DirectionalLight uLight0;

uniform int uNormalMapping;

void main() {
	vec3 diffuseColor = uMaterial.diffuse;
	float alpha = 1.0;
	if (uMaterial.useDiffuseMap > 0){
		vec4 textureColor = texture(uMaterial.diffuseMap, fs_in.uv);
		diffuseColor = textureColor.rgb;
		alpha = textureColor.a;	
	}
	
	vec3 tsNormal = vec3(0, 0, 1);
	if (uNormalMapping > 0 && uMaterial.useNormalMap > 0){
		tsNormal = normalize(texture(uMaterial.normalMap, fs_in.uv).rgb * 2.0 - 1.0);
	}
	vec3 tsToLight = normalize(fs_in.tsToLight);
	vec3 tsToCamera = normalize(fs_in.tsToCamera);

	vec3 ambient = uLight0.ambient * uMaterial.ambient * diffuseColor;

	float diffuseFactor = clamp(dot(tsNormal, tsToLight), 0, 1);
	vec3 diffuse = uLight0.diffuse * diffuseFactor * diffuseColor;

	vec3 reflected = reflect(-tsToLight, tsNormal);
	float specularFactor = pow(clamp(dot(tsToCamera, reflected), 0, 1), uMaterial.shininess);
	vec3 specular = uLight0.specular * specularFactor * uMaterial.specular;	

	vec3 sceneColor = ambient + diffuse + specular;
	float v = clamp(
		exp(-pow(length(fs_in.vsPosition) * uFog.density, uFog.gradient)),
		0, 1);
	outColor = vec4(sceneColor * v + uFog.color * (1.0 - v), 1.0);
	outDepth = vec4(length(fs_in.vsPosition), 0, 0, 1);
}