#version 330

layout (location = 0) in vec3 aPosition;
layout (location = 1) in vec2 aUv;
layout (location = 2) in vec3 aNormal;
layout (location = 3) in vec3 aTangent;

out VS_OUT {
	vec3 vsPosition;
	vec2 uv;
	vec3 tsToLight;
	vec3 tsToCamera;
} vs_out;

// light
struct DirectionalLight {
	vec3 direction;
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};
uniform DirectionalLight uLight0;

uniform vec3 uCameraPosition;
uniform mat4 uModelMatrix;
uniform mat4 uViewMatrix;
uniform mat4 uProjectionMatrix;

uniform vec4 uClipPlane;

void main() {
	vec4 wsPosition = uModelMatrix * vec4(aPosition, 1.0);
	vec4 vsPosition = uViewMatrix * wsPosition;
	gl_Position = uProjectionMatrix * vsPosition;

	gl_ClipDistance[0] = dot(wsPosition, uClipPlane);

	vs_out.vsPosition = vsPosition.xyz;
	vs_out.uv = aUv;
	vs_out.uv.x = 1.0 - vs_out.uv.x;

	mat4 normalMatrix = transpose(inverse(uModelMatrix));
	vec3 N = normalize(vec3(normalMatrix * vec4(aNormal, 0)));
	vec3 T = normalize(vec3(normalMatrix * vec4(aTangent, 0)));
	T = normalize(T - dot(T, N) * N); // re-orthogonalize T with respect to N
	vec3 B = normalize(cross(N, T));
	mat3 TBN = transpose(mat3(T, B, N));

	vs_out.tsToLight = TBN * normalize(-uLight0.direction);	
	vs_out.tsToCamera = TBN * normalize(uCameraPosition - wsPosition.xyz);
}