#version 330

layout(location = 0) out vec4 outColor;
layout(location = 1) out vec4 outBloom;

in GS_OUT{
	vec3 vsPosition;
	vec4 csPosition;
	vec4 uvs;
	float blend;
	float alpha;
} fs_in;

uniform sampler2D uDiffuseMap;
uniform vec3 uDiffuseIntensity;
uniform vec3 uBloomIntensity;
uniform int uUseBloom;

uniform sampler2D uDepth;
uniform int uUseDepth;

void main() {
	vec4 diffuseColor = mix(
		texture2D(uDiffuseMap, fs_in.uvs.xy), 
		texture2D(uDiffuseMap, fs_in.uvs.zw), 
		fs_in.blend
	);

	float depthAlpha = 1.0;
	float softScale = 0.05;
	if (uUseDepth > 0) {
		vec2 uv = (fs_in.csPosition.xy / fs_in.csPosition.w) * 0.5 + 0.5;
		float sceneDepth = texture2D(uDepth, uv).r;
		float partDepth = length(fs_in.vsPosition);
		depthAlpha = clamp(abs(sceneDepth - partDepth) * softScale, 0.0, 1.0);
	}

	diffuseColor.a *= fs_in.alpha * depthAlpha;
	outColor = diffuseColor * vec4(uDiffuseIntensity, 1.0);
	outBloom = uUseBloom > 0 ? 
		diffuseColor * vec4(uBloomIntensity, 1.0) : vec4(0);
}