#version 330

layout(points) in;
layout(triangle_strip, max_vertices = 4) out;

in VS_OUT{
	vec3 position;
	vec2 life;
	vec3 transform;
} gs_in[];

out GS_OUT{
	vec3 vsPosition;
	vec4 csPosition;
	vec4 uvs;
	float blend;
	float alpha;
} gs_out;

uniform int uGridCount;
uniform vec2 uFading;

uniform mat4 uViewMatrix;
uniform mat4 uProjectionMatrix;

vec2 getOffset(int index) {
	float atlasGrids = float(uGridCount);
	int g = int(atlasGrids);
	int i = clamp(index, 0, g * g - 1);
	return vec2(
		float(int(mod(i, g))) / atlasGrids,
		float(i / g) / atlasGrids
	);
}

void main() {
	if (gs_in[0].life.y > 0) {

		float lifeProgress = min(1.0, gs_in[0].life.x / gs_in[0].life.y);
		float indexProgress = lifeProgress * float(uGridCount * uGridCount);
		gs_out.blend = indexProgress - float(int(indexProgress));
		int index = int(indexProgress);
		vec2 uvs1 = getOffset(index);
		vec2 uvs2 = getOffset(index + 1);

		gs_out.alpha = 1.0;
		float thrIn = clamp(uFading.x, 0.0, uFading.y);
		float thrOut = clamp(uFading.y, uFading.x, 1.0);
		if (lifeProgress < thrIn)
			gs_out.alpha *= (lifeProgress / thrIn);
		if (lifeProgress > thrOut)
			gs_out.alpha *= (1.0 - ((lifeProgress - thrOut) / (1.0 - thrOut)));

		float scaling = max(0.0,
			(1.0 - lifeProgress) * gs_in[0].transform.x
			+ lifeProgress * gs_in[0].transform.y);
		float rot = gs_in[0].transform.z;
		mat4 modelMatrix = mat4(
			cos(rot), -sin(rot), 0, 0,
			sin(rot), cos(rot), 0, 0,
			0, 0, 1, 0,
			0, 0, 0, 1
		);

		vec4 vsPosition = uViewMatrix * vec4(gs_in[0].position, 1.0);
		float ig = 1.0 / float(uGridCount);


		vec4 vpos = modelMatrix * vec4(-scaling, scaling, 0, 0) + vsPosition;
		vec4 ppos = uProjectionMatrix * vpos;
		gs_out.vsPosition = vpos.xyz;
		gs_out.csPosition = ppos;
		gl_Position = ppos;
		gs_out.uvs.xy = uvs1;
		gs_out.uvs.zw = uvs2;
		EmitVertex();

		vpos = modelMatrix * vec4(-scaling, -scaling, 0, 0) + vsPosition;
		ppos = uProjectionMatrix * vpos;
		gs_out.vsPosition = vpos.xyz;
		gs_out.csPosition = ppos;
		gl_Position = ppos;
		gs_out.uvs.xy = vec2(0, ig) + uvs1;
		gs_out.uvs.zw = vec2(0, ig) + uvs2;
		EmitVertex();

		vpos = modelMatrix * vec4(scaling, scaling, 0, 0) + vsPosition;
		ppos = uProjectionMatrix * vpos;
		gs_out.vsPosition = vpos.xyz;
		gs_out.csPosition = ppos;
		gl_Position = ppos;
		gs_out.uvs.xy = vec2(ig, 0) + uvs1;
		gs_out.uvs.zw = vec2(ig, 0) + uvs2;
		EmitVertex();

		vpos = modelMatrix * vec4(scaling, -scaling, 0, 0) + vsPosition;
		ppos = uProjectionMatrix * vpos;
		gs_out.vsPosition = vpos.xyz;
		gs_out.csPosition = ppos;
		gl_Position = ppos;
		gs_out.uvs.xy = vec2(ig, ig) + uvs1;
		gs_out.uvs.zw = vec2(ig, ig) + uvs2;
		EmitVertex();

		EndPrimitive();
	}
}