#version 330

layout(points) in;
layout(points, max_vertices = 50) out;

in VS_OUT{
	vec3 position;
	vec2 life;
	vec3 transform;
	vec3 velocity;
} gs_in[];

out vec3 position;
out vec2 life;
out vec3 transform;
out vec3 velocity;

uniform float uDeltaTime;
uniform int uParticlesToSpawn;
uniform float uTime;
uniform float uSpawnLife;
uniform vec3 uSpawnPosition;
uniform vec3 uSpawnPositionError;
uniform vec3 uSpawnVelocity;
uniform vec3 uSpawnVelocityError;
uniform vec3 uSpawnTransform;
uniform vec3 uSpawnTransformError;
uniform float uMass;
//uniform vec3 uGravity;

// RANDOM FUNCTIONS
uint seed;

float randf() {
	uint i = (seed ^ 12345391u) * 2654435769u;
	i ^= (i << 6u) ^ (i >> 26u);
	i *= 2654435769u;
	i += (i << 5u) ^ (i >> 12u);
	seed += 1u;
	return float(i) / 4294967295.0;
}

float randf(float lo, float hi) {
	if (lo == hi) {
		return lo;
	}
	if (lo > hi) {
		float temp = lo;
		lo = hi;
		hi = temp;
	}
	return lo + randf() * (hi - lo);
}

float randfSimulation(float sim, float error) {
	return randf(sim - error, sim + error);
}

vec2 randfSimulation2(vec2 sim, vec2 error) {
	return vec2(
		randfSimulation(sim.x, error.x),
		randfSimulation(sim.y, error.y)
	);
}

vec3 randfSimulation3(vec3 sim, vec3 error) {
	return vec3(
		randfSimulation(sim.x, error.x),
		randfSimulation(sim.y, error.y),
		randfSimulation(sim.z, error.z)
	);
}

vec3 randDirection3() {
	return normalize(vec3(
		randf(-1.0, 1.0),
		randf(-1.0, 1.0),
		randf(-1.0, 1.0)
	));
}

void main() {
	// initialize seed
	seed = uint(uTime * 1000.0);

	if (gs_in[0].life.y == -1.0) {
		// Emitter
		position = vec3(0, 0, 0);
		life = vec2(0, -1.0);
		transform = vec3(0, 0, 0);
		velocity = vec3(0, 0, 0);
		EmitVertex();
		EndPrimitive();

		// Spawn new particle
		for (int i = 0; i < uParticlesToSpawn; i++) {
			position = randfSimulation3(uSpawnPosition, uSpawnPositionError);
			life = vec2(0, uSpawnLife);
			transform = randfSimulation3(uSpawnTransform, uSpawnTransformError);
			velocity = randfSimulation3(uSpawnVelocity, uSpawnVelocityError);
			EmitVertex();
			EndPrimitive();
		}
	}
	else if (gs_in[0].life.y > 0){
		// Update particle
		float elapsed = gs_in[0].life.x + uDeltaTime;
		if (elapsed < gs_in[0].life.y) {
			position = gs_in[0].position + uDeltaTime * gs_in[0].velocity;
			life = vec2(elapsed, gs_in[0].life.y);
			transform = gs_in[0].transform;
			velocity = gs_in[0].velocity + uMass * vec3(0, -30, 0) * uDeltaTime;

			EmitVertex();
			EndPrimitive();
		}
	}
}