#version 410

layout(triangles, equal_spacing, ccw) in;

in CS_OUT{
	vec3 wsPosition;
	vec3 vsPosition;
	vec2 uv;
	vec3 wsNormal;
	vec3 wsTangent;
	float tessellationRatio;
} es_in[];

out ES_OUT{
	vec3 vsPosition;
	vec2 uv;
	vec3 tsToLight;
	vec3 tsToCamera;
	vec4 smPosition;
} fs_in;

// light
struct DirectionalLight {
	vec3 direction;
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};
uniform DirectionalLight uLight0;

// material
struct Material {
	sampler2D maskMap;

	sampler2D displacementMap;
	float displacementStrength;

	float tilingR;
	vec3 ambientR;
	sampler2D diffuseMapR;
	vec3 specularR;
	float shininessR;
	sampler2D normalMapR;
	int useNormalMapR;

	float tilingG;
	vec3 ambientG;
	sampler2D diffuseMapG;
	vec3 specularG;
	float shininessG;
	sampler2D normalMapG;
	int useNormalMapG;

	float tilingB;
	vec3 ambientB;
	sampler2D diffuseMapB;
	vec3 specularB;
	float shininessB;
	sampler2D normalMapB;
	int useNormalMapB;
};
uniform Material uMaterial;

uniform vec3 uCameraPosition;
uniform mat4 uViewMatrix;
uniform mat4 uProjectionMatrix;

uniform mat4 uShadowMapMVP;

uniform int uDisplacementMapping;
uniform vec4 uClipPlane;

float interpolate1D(float v0, float v1, float v2)
{
	return gl_TessCoord.x * v0 + gl_TessCoord.y * v1 + gl_TessCoord.z * v2;
}

vec2 interpolate2D(vec2 v0, vec2 v1, vec2 v2)
{
	return gl_TessCoord.x * v0 + gl_TessCoord.y * v1 + gl_TessCoord.z * v2;
}

vec3 interpolate3D(vec3 v0, vec3 v1, vec3 v2)
{
	return gl_TessCoord.x * v0 + gl_TessCoord.y * v1 + gl_TessCoord.z * v2;
}

void main() {
	vec4 wsPosition = vec4(interpolate3D(es_in[0].wsPosition, es_in[1].wsPosition, es_in[2].wsPosition), 1.0);
	vec4 vsPosition = vec4(interpolate3D(es_in[0].vsPosition, es_in[1].vsPosition, es_in[2].vsPosition), 1.0);
	fs_in.uv = interpolate2D(es_in[0].uv, es_in[1].uv, es_in[2].uv);
	vec3 N = normalize(interpolate3D(es_in[0].wsNormal, es_in[1].wsNormal, es_in[2].wsNormal)); 
	vec3 T = normalize(interpolate3D(es_in[0].wsTangent, es_in[1].wsTangent, es_in[2].wsTangent));
	float tessellationRatio = interpolate1D(es_in[0].tessellationRatio, es_in[1].tessellationRatio, es_in[2].tessellationRatio);
	
	gl_ClipDistance[0] = dot(wsPosition, uClipPlane);

	// displacement mapping
	float range = 25.0;
	float distance = length(vsPosition.xyz);
	if (uDisplacementMapping > 0 && tessellationRatio > 0 && distance < range) {
		float displacement = texture2D(uMaterial.displacementMap, fs_in.uv * uMaterial.tilingB).b * 
			uMaterial.displacementStrength;
		wsPosition.xyz += N * displacement * clamp(1.0 - distance / range, 0, 1);
	}

	vsPosition = uViewMatrix * wsPosition;
	fs_in.vsPosition = vsPosition.xyz;
	T = normalize(T - dot(T, N) * N); // re-orthogonalize T with respect to N
	vec3 B = normalize(cross(N, T));
	mat3 TBN = transpose(mat3(T, B, N));

	fs_in.tsToLight = TBN * normalize(-uLight0.direction);
	fs_in.tsToCamera = TBN * normalize(uCameraPosition - wsPosition.xyz);
	fs_in.smPosition = uShadowMapMVP * wsPosition;

	gl_Position = uProjectionMatrix * vsPosition;
}