#version 330

layout(location = 0) in vec3 aPosition;
layout(location = 1) in vec2 aUv;
layout(location = 2) in vec3 aNormal;
layout(location = 3) in vec3 aTangent;

out VS_OUT{
	vec3 wsPosition;
	vec3 vsPosition;
	vec2 uv;
	vec3 wsNormal;
	vec3 wsTangent;
	float tessellationRatio;
} cs_in;

// material
struct Material {
	sampler2D maskMap;

	sampler2D displacementMap;
	float displacementStrength;

	float tilingR;
	vec3 ambientR;
	sampler2D diffuseMapR;
	vec3 specularR;
	float shininessR;
	sampler2D normalMapR;
	int useNormalMapR;

	float tilingG;
	vec3 ambientG;
	sampler2D diffuseMapG;
	vec3 specularG;
	float shininessG;
	sampler2D normalMapG;
	int useNormalMapG;

	float tilingB;
	vec3 ambientB;
	sampler2D diffuseMapB;
	vec3 specularB;
	float shininessB;
	sampler2D normalMapB;
	int useNormalMapB;
};
uniform Material uMaterial;
uniform mat4 uViewMatrix;

void main() {
	cs_in.wsPosition = aPosition;
	cs_in.vsPosition = vec3(uViewMatrix * vec4(aPosition, 1.0));
	cs_in.uv = aUv;
	cs_in.wsNormal = normalize(aNormal);
	cs_in.wsTangent = normalize(aTangent);
	cs_in.tessellationRatio = texture2D(uMaterial.maskMap, aUv).b;
}