#version 330

layout (location = 0) in vec3 aPosition;
layout (location = 1) in vec2 aUv;
layout (location = 2) in vec3 aNormal;
layout (location = 3) in vec3 aTangent;

out VS_OUT {
	vec3 wsPosition;
	vec3 vsPosition;
	vec4 csPosition;
	vec2 uv;
	vec3 tsToLight;
	vec3 tsToCamera;
} vs_out;

// light
struct DirectionalLight {
	vec3 direction;
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};
uniform DirectionalLight uLight0;

uniform vec3 uCameraPosition;
uniform mat4 uViewMatrix;
uniform mat4 uProjectionMatrix;

uniform float uHeight;

void main() {
	vec4 wsPosition = vec4(aPosition, 1.0);
	wsPosition.y = uHeight;
	vec4 vsPosition = uViewMatrix * wsPosition;

	vec3 N = normalize(aNormal);
	vec3 T = normalize(aTangent);
	T = normalize(T - dot(T, N) * N); // re-orthogonalize T with respect to N
	vec3 B = normalize(cross(N, T));
	mat3 TBN = transpose(mat3(T, B, N));

	vs_out.wsPosition = wsPosition.xyz;
	vs_out.vsPosition = vsPosition.xyz;
	vs_out.csPosition = uProjectionMatrix * vsPosition;
	vs_out.uv = aUv * 70.0;
	vs_out.tsToLight = TBN * normalize(-uLight0.direction);
	vs_out.tsToCamera = TBN * normalize(uCameraPosition - wsPosition.xyz);
	
	gl_Position = vs_out.csPosition;
}